package com.survey7.flashcards;

/*
Copyright 2012 Sanjay Rao, Seattle WA USA

This file is part of Flash Cards GPL.

Flash Cards GPL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

Flash Cards GPL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Flash Cards GPL.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Collections;
import java.util.Arrays;
import android.os.Handler;
import android.content.res.Configuration;
import android.app.Activity;
import android.os.Bundle;
import android.text.Html;
import android.view.GestureDetector;
import android.view.GestureDetector.SimpleOnGestureListener;
import android.view.MotionEvent;
import android.view.Window;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Button;
import android.view.ViewGroup;
import android.util.Log;
import java.lang.StringBuilder;
import android.text.SpannableString;
import android.text.style.UnderlineSpan;
import android.graphics.Color;
import android.view.MenuInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.content.Intent;
import android.app.AlertDialog;
import android.content.DialogInterface;
import java.io.UnsupportedEncodingException;
import android.net.Uri;
import java.net.URLEncoder;
import java.util.ArrayList;

public class FlashCards extends Activity {
	public boolean isforward=true;
	private static final String DBG="FlashCards";
	private Globals globals;
	private ArrayList<String> wikistringlist=new ArrayList<String>();
	private cardside front,back,buttons,nullside,curside;
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		front.setactivity(this);
		back.setactivity(this);
	}
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater MI=getMenuInflater();
		MI.inflate(R.menu.cardmenu,menu);
		return true;
	}
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.options_menu:
				Intent intent=new Intent();
				intent.setClass(this,Options.class);
				startActivity(intent);
				break;
			case R.id.links_menu:
				showwikilinks();
				break;
		}
		return false;
	}
	@Override
	public boolean onSearchRequested() {
		showwikilinks();
		return false;
	}
	@Override
	public void onCreate(Bundle savedInstanceState) {
		globals=(Globals)((Globalcontainer)getApplicationContext()).globals;
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		super.onCreate(savedInstanceState);
		nullside=new nullside();
		front=new front();
		back=new back();
		buttons=new buttonside();
		curside=nullside;
	}
	@Override
	public void onResume() {
		super.onResume();
		front.setactivity(this);
		back.setactivity(this);
		buttons.setactivity(this);
		curside=nullside;
		drawcard();
	}
	@Override
	public void onStop() {
		super.onStop();
		if (curside==null) return;
// this is necessary in case there's a postDelayed runnable that wants to crash us
		curside.unsetup();
		curside=null;
	}
	private void wikilookup(String link) {
		String ustr,url;
		try { ustr=URLEncoder.encode(link,"UTF-8"); }
		catch(UnsupportedEncodingException e) { ustr=""; };
		url="http://en.wikipedia.org/wiki/Special:Search?search="+ustr;
		Intent intent=new Intent(Intent.ACTION_VIEW);
		intent.setData(Uri.parse(url));
		startActivity(intent);
	}
	private void showwikilinks() {
		AlertDialog.Builder ab;
		String[] items;
		ab=new AlertDialog.Builder(FlashCards.this);
		ab.setTitle(R.string.longwikilinks);
		items=wikistringlist.toArray(new String[0]);
		if (items.length==0) { items=new String[1]; items[0]="No links on this display"; }
		ab.setItems(items,new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int buttonindex) {
					if (buttonindex>=wikistringlist.size()) return;
					wikilookup(wikistringlist.get(buttonindex));
				}
			});
		ab.setNegativeButton(R.string.cancel,new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int ign) {
				}
			});
		ab.create();
		ab.show();
	}
	private void drawfront(card card) {
		if (curside!=front) {
			curside.unsetup();
			curside=front;
			curside.setup();
		}
		curside.drawcard(card,false);
	}
	private void drawbuttons(card card) {
		if (curside!=buttons) {
			curside.unsetup();
			curside=buttons;
			curside.setup();
		}
		curside.drawcard(card,false);
	}
	private void drawback(card card, boolean denyfacts) {
		if (curside!=back) {
			curside.unsetup();
			curside=back;
			curside.setup();
		}
		curside.drawcard(card,denyfacts);
	}
	private void drawcard() {
		card card=globals.quiz.getcurrent();
		wikistringlist.clear();
		if (globals.quiz.cursor>globals.quiz.lastback) {
			drawfront(card);
		} else drawback(card,false);
	}
	public void showbuttons() {
		wikistringlist.clear();
		if (globals.multiplechoice) drawbuttons(globals.quiz.getcurrent());
		else drawback(globals.quiz.getcurrent(),true);
	}
	public void wronganswer() {
		globals.quiz.setcorrect(false);
	}
	public void repeatcard() {
		globals.quiz.repeatcard();
		nextcard();
	}
	public void addwikilink(String in) {
		if (in==null) return;
		if (in.equals("")) return;
		wikistringlist.add(in);
	}
	public void nextcard() {
		quiz quiz=globals.quiz;
		if (quiz.islastcard()) finish();
		else quiz.nextcard(globals.repeatmissed);
		isforward=true;
		drawcard();
		System.gc();
	}
	public void prevcard() {
		globals.quiz.prevcard();
		isforward=false;
		drawcard();
	}
}
