package com.survey7.flashcards;

/*
Copyright 2012 Sanjay Rao, Seattle WA USA

This file is part of Flash Cards GPL.

Flash Cards GPL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

Flash Cards GPL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Flash Cards GPL.  If not, see <http://www.gnu.org/licenses/>.
*/

import android.app.Activity;
import android.content.res.Configuration;
import android.os.Bundle;
import android.content.Intent;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.view.View;
import android.widget.TextView;
import android.widget.Button;
import android.widget.RadioButton;
import android.util.Log;
import android.content.SharedPreferences;
import android.widget.LinearLayout;
import android.view.ViewGroup;
import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import android.widget.Spinner;

public class Options extends Activity {
	private static final String DBG="FlashCards";
	private Globals globals;
	static final int[] allids={
			R.id.repeat_options,
			R.id.norepeat_options,
			R.id.simple_options,
			R.id.all_options,
			R.id.cards1of2_options,
			R.id.cards2of2_options,
			R.id.random_options,
			R.id.forward_options,
			R.id.reverse_options,
			R.id.facts_options,
			R.id.nofacts_options,
			R.id.white_options,
			R.id.blue_options,
			R.id.whiteonblack_options,
			R.id.blackonwhite_options,
			R.id.redonblack_options,
			R.id.animate_options,
			R.id.noanimate_options,
			R.id.showbuiltins_options,
			R.id.noshowbuiltins_options,
			R.id.vibrate_options,
			R.id.novibrate_options
		};

	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
	}
	@Override
	public void onCreate(Bundle savedInstanceState) {
		globals=(Globals)((Globalcontainer)getApplicationContext()).globals;
		globals.init(this);
		super.onCreate(savedInstanceState);
		setContentView(R.layout.options);
		setradios();
		for (int i=0;i<allids.length;i++) {
			((RadioButton)findViewById(allids[i])).setOnClickListener(new View.OnClickListener() { public void onClick(View v) { radioclick(v); } });
		}
	}
	private void setradio(int id) {
		((RadioButton)findViewById(id)).toggle();
	}
	private void setcolors() {
		if (globals.frontjpeg.equals("white")
				&& globals.fgcolor==globals.black
				&& globals.bgcolor==globals.white) setradio(R.id.white_options);
		else if (globals.frontjpeg.equals("blue")
				&& globals.fgcolor==globals.black
				&& globals.bgcolor==globals.blue) setradio(R.id.blue_options);
		else if (globals.frontjpeg.equals("")) {
				if (globals.fgcolor==globals.white
					&& globals.bgcolor==globals.black) setradio(R.id.whiteonblack_options);
				else if (globals.fgcolor==globals.black
					&& globals.bgcolor==globals.white) setradio(R.id.blackonwhite_options);
				else if (globals.fgcolor==globals.red
					&& globals.bgcolor==globals.black) setradio(R.id.redonblack_options);
		}
	}
	private void setradios() {
		setcolors();
		switch (globals.uimode) {
			case 0: setradio(R.id.repeat_options); break;
			case 1: setradio(R.id.norepeat_options); break;
			case 2: setradio(R.id.simple_options); break;
		}
		switch (globals.subset) {
			case 0: setradio(R.id.all_options); break;
			case 1: setradio(R.id.cards1of2_options); break;
			case 2: setradio(R.id.cards2of2_options); break;
		}
		switch (globals.direction) {
			case 0: setradio(R.id.random_options); break;
			case 1: setradio(R.id.forward_options); break;
			case 2: setradio(R.id.reverse_options); break;
		}
		switch (globals.facts) {
			case 0: setradio(R.id.facts_options); break;
			case 1: setradio(R.id.nofacts_options); break;
		}
		switch (globals.doanimation) {
			case 0: setradio(R.id.noanimate_options); break;
			case 1: setradio(R.id.animate_options); break;
		}
		if (globals.showbuiltins) setradio(R.id.showbuiltins_options);
		else setradio(R.id.noshowbuiltins_options);
		if (globals.denyvibrate) setradio(R.id.novibrate_options);
		else setradio(R.id.vibrate_options);
	}
	private void radioclick(View v) {
		switch (v.getId()) {
			case R.id.repeat_options: // Multiple choice, repeat missed
				globals.uimode=0;
				break;
			case R.id.norepeat_options: // Multiple choice, don't repeat
				globals.uimode=1;
				break;
			case R.id.simple_options: // Traditional front/back
				globals.uimode=2;
				break;
			case R.id.all_options: // All cards
				globals.subset=0;
				break;
			case R.id.cards1of2_options: // 1,3,5,...
				globals.subset=1;
				break;
			case R.id.cards2of2_options: // 2,4,6,...
				globals.subset=2;
				break;
			case R.id.random_options: // randomize
				globals.direction=0;
				break;
			case R.id.forward_options: // forward, no random
				globals.direction=1;
				break;
			case R.id.reverse_options: // reversed order
				globals.direction=2;
				break;
			case R.id.facts_options: // show related facts on back
				globals.facts=0;
				break;
			case R.id.nofacts_options: // don't show facts
				globals.facts=1;
				break;
			case R.id.white_options: // white index card
				globals.fgcolor=globals.black;
				globals.bgcolor=globals.white;
				globals.frontjpeg="white";
				break;
			case R.id.blue_options: // blue index card
				globals.fgcolor=globals.black;
				globals.bgcolor=globals.blue;
				globals.frontjpeg="blue";
				break;
			case R.id.whiteonblack_options: // white on black
				globals.fgcolor=globals.white;
				globals.bgcolor=globals.black;
				globals.frontjpeg="";
				break;
			case R.id.blackonwhite_options: // black on white
				globals.fgcolor=globals.black;
				globals.bgcolor=globals.white;
				globals.frontjpeg="";
				break;
			case R.id.redonblack_options: // red on black
				globals.fgcolor=globals.red;
				globals.bgcolor=globals.black;
				globals.frontjpeg="";
				break;
			case R.id.animate_options: // left/right slides
				globals.doanimation=1;
				break;
			case R.id.noanimate_options: // no animation
				globals.doanimation=0;
				break;
			case R.id.showbuiltins_options: // show built-in cards
				globals.showbuiltins=true;
				break;
			case R.id.noshowbuiltins_options: // don't show built-in cards
				globals.showbuiltins=false;
				break;
			case R.id.vibrate_options: // allow vibration
				globals.denyvibrate=false;
				break;
			case R.id.novibrate_options: // deny vibration
				globals.denyvibrate=true;
				break;
		}
		globals.savepreferences();
	}
}
