package com.survey7.flashcards;

/*
Copyright 2012 Sanjay Rao, Seattle WA USA

This file is part of Flash Cards GPL.

Flash Cards GPL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

Flash Cards GPL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Flash Cards GPL.  If not, see <http://www.gnu.org/licenses/>.
*/

import android.util.Log;
import org.json.JSONObject;
import org.json.JSONArray;
import org.json.JSONException;
import java.lang.StringBuilder;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.util.TreeMap;
import java.util.TreeSet;

public class buildquiz {
	private static final String DBG="FlashCards";
	TreeMap<String,rawimage> bitmapsbyfn=new TreeMap<String,rawimage>();
	TreeSet<String> missingfilenames=new TreeSet<String>();
	quiz quiz=new quiz();
	JSONObject json;
	notion[] notions;
	class fact {
		notion frontn;
		String dim;
		notion[] backns;
	}
	fact[] facts;

	private notion makenotion(String str, String filename, String hint) {
		notion ret=new notion();
		ret.string=str;
		if (!filename.equals("")) {
			rawimage bmp=bitmapsbyfn.get(filename);
			if (bmp==null) missingfilenames.add(filename);
			else ret.bitmap=bmp;
		}
		if (!hint.equals("")) {
			rawimage bmp=bitmapsbyfn.get(hint);
			if (bmp==null) missingfilenames.add(hint);
			else ret.hint=bmp;
		}
		return ret;
	}

	private void loadstrings() throws org.json.JSONException {
		JSONArray a;
		int i,n;
		a=json.getJSONArray("strings");
		n=a.length();
		notions=new notion[n];
		for (i=0;i<n;i++) {
			JSONObject o;
			String s;
			o=a.optJSONObject(i);
			if (o==null) {
				s=a.getString(i);
				notions[i]=makenotion(s,"","");
			} else {
				String img,hint;
				s=o.optString("s"); // returns "" on failure
				hint=o.optString("h");
				img=o.optString("i");
				notions[i]=makenotion(s,img,hint);
			}
		}
	}
	private void loadfact(fact dest, JSONArray a) throws org.json.JSONException {
		int i,n;
		int fint;
		n=a.length();
		fint=a.getInt(0);
		dest.frontn=notions[fint];
		dest.dim=notions[a.getInt(1)].string;
		dest.backns=new notion[n-2];
		for (i=2;i<n;i++) dest.backns[i-2]=notions[a.getInt(i)];
	}
	private void loadfacts() throws org.json.JSONException {
		JSONArray a;
		int i,n;
		a=json.getJSONArray("facts");
		n=a.length();
		facts=new fact[n];
		for (i=0;i<n;i++) {
			fact f=new fact();
			facts[i]=f;
			loadfact(f,a.getJSONArray(i));
		}
	}
	private void buildwrongs(card dest, JSONArray wrongs) throws org.json.JSONException {
		int i,n=wrongs.length();
		dest.wrongns=new notion[n];
		for (i=0;i<n;i++) dest.wrongns[i]=notions[wrongs.getInt(i)];
	}
	private String buildquestion(fact fact) {
		String str=fact.dim;
		int pos;
		pos=str.indexOf("$1");
		if (pos<0) {
			return null;
		} else {
			StringBuilder sb=new StringBuilder();
			sb.append(str.substring(0,pos));
			sb.append(fact.frontn.string);
			sb.append(str.substring(pos+2,str.length()));
			return sb.toString();
		}
	}
	private notion buildfact(fact fact) {
		String str=fact.dim;
		int pos;
		notion ret=new notion();
		String front=fact.frontn.string;

		ret.bitmap=fact.frontn.bitmap;
		pos=str.indexOf("$1");
		if (pos<0) {
			ret.string=fact.dim+": "+front;
		} else {
			StringBuilder sb=new StringBuilder();
			sb.append(str.substring(0,pos));
			sb.append(front);
			sb.append(str.substring(pos+2,str.length()));
			ret.string=sb.toString();
		}
		return ret;
	}
	private void buildfacts(card dest, JSONArray a) throws org.json.JSONException {
		if (a==null) {
			dest.facts=new notion[0];
			return;
		}
		int i,n=a.length();
		dest.facts=new notion[n];
		for (i=0;i<n;i++) dest.facts[i]=buildfact(facts[a.getInt(i)]);
	}
	private void buildcard(card dest,JSONObject src) throws org.json.JSONException {
		fact f;
		f=facts[src.getInt("f")];
		dest.leftn=f.frontn;
		dest.dim=f.dim;
		dest.rightns=f.backns;
		dest.question=buildquestion(f);
		buildwrongs(dest,src.getJSONArray("w"));
		buildfacts(dest,src.optJSONArray("x"));
	}
	private void buildcards() throws org.json.JSONException {
		int i,n;
		JSONArray a;
		a=json.getJSONArray("cards");
		n=a.length();
		quiz.cards=new card[n];
		for (i=0;i<n;i++) {
			card card=new card();
			quiz.cards[i]=card;
			buildcard(card,a.getJSONObject(i));
		}
	}
	private void parsequizjson(String data) throws org.json.JSONException {
		json=new JSONObject(data);
		quiz.title=json.getString("title");
		loadstrings();
		loadfacts();
		buildcards();
	}
	private void addimage(String fn, InputStream is) {
		rawimage bmp;
		bmp=new rawimage(datautils.slurpstream2(is));
		if (bmp==null) {
			Log.w(DBG,"Error decoding image "+fn);
			return;
		}
// Log.i(DBG,"Success decoding image "+fn);
		bitmapsbyfn.put(fn,bmp);
	}
	private quiz load2(InputStream is) throws java.io.IOException {
		ZipInputStream zis=new ZipInputStream(new BufferedInputStream(is));
		ZipEntry ze;
		String json=null;
		quiz ret=null;
		while (true) {
			ze=zis.getNextEntry();
			if (ze==null) break;
			String fn;
			fn=ze.getName();
			if (fn.equals("quiz.json")) json=datautils.slurpstream(zis);
			else if (!fn.endsWith(".txt")) addimage(fn,zis);
		}
		zis.close();
		if (json!=null)  try {
				parsequizjson(json);
				quiz.missingfilenames=missingfilenames.toArray(new String[0]);
				ret=quiz;
		} catch (org.json.JSONException e) { Log.e(DBG,"JSONException"); }
		return ret;
	}
	public static quiz load(InputStream is) {
		quiz ret=null;
		buildquiz bq=new buildquiz();
		try { ret=bq.load2(is); } catch(java.io.IOException e){}
		return ret;
	}
}
